library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity vga_out_v1_0 is
    generic (
        C_COLOR_IN_WIDTH : natural range 4 to 12 := 8;
        C_COLOR_OUT_WIDTH : natural range 4 to 12 := 4
    );
    port (
        clk     : in  std_logic;

        vid_in  : in  std_logic_vector((C_COLOR_IN_WIDTH*3)-1 downto 0);
        vs_in   : in  std_logic;
        hs_in   : in  std_logic;
        de_in   : in  std_logic;

        r_out   : out std_logic_vector(C_COLOR_OUT_WIDTH-1 downto 0);
        g_out   : out std_logic_vector(C_COLOR_OUT_WIDTH-1 downto 0);
        b_out   : out std_logic_vector(C_COLOR_OUT_WIDTH-1 downto 0);
        vs_out  : out std_logic;
        hs_out  : out std_logic
    );
end vga_out_v1_0;

architecture arch_imp of vga_out_v1_0 is

    constant C_WIDTH_DIFF : NATURAL := C_COLOR_IN_WIDTH-C_COLOR_OUT_WIDTH;

    signal s_r : std_logic_vector(C_COLOR_IN_WIDTH-1 downto 0);
    signal s_g : std_logic_vector(C_COLOR_IN_WIDTH-1 downto 0);
    signal s_b : std_logic_vector(C_COLOR_IN_WIDTH-1 downto 0);

begin

    s_r <= vid_in((C_COLOR_IN_WIDTH*3)-1 downto (C_COLOR_IN_WIDTH*2));
    s_g <= vid_in( C_COLOR_IN_WIDTH   -1 downto  0);
    s_b <= vid_in((C_COLOR_IN_WIDTH*2)-1 downto (C_COLOR_IN_WIDTH));

    p_out : process(clk) is
    begin
        if de_in = '1' then
            r_out <= s_r(C_COLOR_IN_WIDTH-1 downto C_COLOR_OUT_WIDTH);
            g_out <= s_g(C_COLOR_IN_WIDTH-1 downto C_COLOR_OUT_WIDTH);
            b_out <= s_b(C_COLOR_IN_WIDTH-1 downto C_COLOR_OUT_WIDTH);
        else
            r_out <= (others => '0');
            g_out <= (others => '0');
            b_out <= (others => '0');
        end if;

        hs_out <= hs_in;
        vs_out <= vs_in;

    end process p_out;


end arch_imp;
