/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: dt_fb.c
 *
 * Date: 03.12.2018
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *      get user frame buffer parameters from devicetree 
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dt_fb.h"


unsigned int be32_to_le32(unsigned int be)
{
    return ((be & 0xFF000000) >> 24) |
           ((be & 0x00FF0000) >> 8)  |
           ((be & 0x0000FF00) << 8)  |
           ((be & 0x000000FF) << 24) ;
}

unsigned int get_dt_fb_params( char *device_path, dt_fb_params_t *params )
{
    unsigned int status = 0;
    FILE *fr;
    char dev_file[80];
    unsigned int tmp;
    
    
    memset(params, 0, sizeof(dt_fb_params_t));

    strcpy(dev_file, device_path);
    strcat(dev_file, "/#address-cells");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_1;
    } else {       
        fread(&tmp, 4, 1, fr);
        params->cells = be32_to_le32(tmp);
        if ( fclose(fr) == EOF ) {
            status |= DT_FB_ERR_2;
        }
    }


    strcpy(dev_file, device_path);
    strcat(dev_file, "/#size-cells");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_3;
    } else {       
        fread(&tmp, 4, 1, fr);
        params->cells_size = be32_to_le32(tmp);
        if ( fclose(fr) == EOF ) {
            status |= DT_FB_ERR_4;
        }
    }

    
    strcpy(dev_file, device_path);
    strcat(dev_file, "/reg");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_5;
    } else {
        if ( params->cells == 2 ) { // ZynqMP
            fread(&tmp, 4, 1, fr);
            params->base_high = be32_to_le32(tmp);
            fread(&tmp, 4, 1, fr);
            params->base_low = be32_to_le32(tmp);
            fread(&tmp, 4, 1, fr);
            params->size_high = be32_to_le32(tmp);
            fread(&tmp, 4, 1, fr);
            params->size_low = be32_to_le32(tmp);            
        } 
        if ( params->cells == 1 ) { //Zynq
            fread(&tmp, 4, 1, fr);
            params->base_low = be32_to_le32(tmp);
            fread(&tmp, 4, 1, fr);
            params->size_low = be32_to_le32(tmp);                    
        }
        if ( fclose(fr) == EOF ) {
            printf("Cannot close file %s\n", dev_file);
            status |= DT_FB_ERR_6;
        }        
    }
       

    strcpy(dev_file, device_path);
    strcat(dev_file, "/width");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_7;
    } else {
        fread(&tmp, 4, 1, fr);
        params->width = be32_to_le32(tmp);
        if ( fclose(fr) == EOF ) {
            status |= DT_FB_ERR_8;
        }        
    }
    
    
    strcpy(dev_file, device_path);
    strcat(dev_file, "/height");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_9;
    } else {
        fread(&tmp, 4, 1, fr);
        params->height = be32_to_le32(tmp);
        if ( fclose(fr) == EOF ) {
            status |= DT_FB_ERR_10;
        }        
    }

    strcpy(dev_file, device_path);
    strcat(dev_file, "/stride");
    if ( (fr = fopen(dev_file, "rb")) == NULL ) {
        status |= DT_FB_ERR_11;
    } else {
        fread(&tmp, 4, 1, fr);
        params->stride = be32_to_le32(tmp);
        if ( fclose(fr) == EOF ) {
            status |= DT_FB_ERR_12;
        }        
    }
    
    return status;
}


void print_dt_fb_params( dt_fb_params_t *params )
{
    printf("cells:      %d\n", params->cells);
    printf("cells_size: %d\n", params->cells_size);
    printf("base_high:  0x%08X\n", params->base_high);
    printf("base_low:   0x%08X\n", params->base_low);
    printf("size_high:  0x%08X\n", params->size_high);
    printf("size_low:   0x%08X\n", params->size_low);
    printf("width:      %d\n", params->width);
    printf("height:     %d\n", params->height);
    printf("stride:     %d\n", params->stride);   
}

